// TradeKeys.cpp : Defines the class behaviors for the application.
//

// TradeKeys Version 2.0
// Copyright  2003 Ziff Davis Media, Inc. All rights reserved.
// First published in PC Magazine, US Edition, January 14, 2003.
// Written by Gregory A. Wolking

#include "stdafx.h"
#include "TradeKeys.h"
#include "TradeKeysDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTradeKeysApp

BEGIN_MESSAGE_MAP(CTradeKeysApp, CWinApp)
	//{{AFX_MSG_MAP(CTradeKeysApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelpFinder)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTradeKeysApp construction

CTradeKeysApp::CTradeKeysApp()
{
	m_pSharedData = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CTradeKeysApp object

CTradeKeysApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CTradeKeysApp initialization

BOOL CTradeKeysApp::InitInstance()
{
	int				iPos;
	CString			strFileName;
	CTradeKeysDlg	dlg;
	HWND			hWndTarget;
	char			szUserName[UNLEN + 1];
	DWORD			dwSize = UNLEN + 1;

	// Create Mutex so Setup knows we're running.
	CreateMutex(NULL, FALSE, "TradeKeysIsRunning");

	/////////////////////////////////
	// Handle filename passed on the command line.
	// Make a copy of the command line.
	strFileName = m_lpCmdLine;
	// Trim leading and trailing blanks.
	strFileName.TrimLeft();
	strFileName.TrimRight();
	// Strip any leading or trailing quotes.
	if (strFileName.Left(1) == "\"")
		strFileName = strFileName.Mid(1);
	if (strFileName.Right(1) == "\"")
		strFileName = strFileName.Left(strFileName.GetLength() - 1);
	//////////////////////////
	// Check Mutex for multiple-instance detection.
	// Try to create or find our mapped data object.
	switch (Init_Mapped_Data())
	{
	case MAP_FIRST:		// We're the first instance, initialized successfully
		break;
	case MAP_SECOND:	// We're the second instance.
		// Copy primary instance's window handle from shared data.
		hWndTarget = m_pSharedData->hWndMain;
		if (hWndTarget == NULL)		// Error if handle is NULL.
		{
			::MessageBox(NULL, "Internal error: hWndMain not defined.", "VolumeSet", MB_ICONSTOP);
			return FALSE;
		}
		if (!IsWindow(hWndTarget))	// Error if handle does not identify an existing window.
		{
			::MessageBox(NULL, "Internal error: hWndMain does not identify an existing window.", "VolumeSet", MB_ICONSTOP);
			return FALSE;
		}
		// Error if that window doesn't belong to another instance of TradeKeys.
		if (::SendMessage(hWndTarget, TRADEKEYS_VERIFY_ID, 0, 0) != TRADEKEYS_APP_ID)
		{
			::MessageBox(NULL, "Internal Error: hWndMain does not\npoint to an instance of VolumeSet", "VolumeSet", MB_ICONSTOP);
			return FALSE;
		}
		// Bring the existing instance to the foreground
		::SetForegroundWindow(hWndTarget);
		if (!strFileName.IsEmpty())
		{
			strcpy(m_pSharedData->pszFileName, strFileName);
			::PostMessage(hWndTarget, TRADEKEYS_OPEN_FILE, 0, 0);
		}
		// And we're done.
		return FALSE;
	case MAP_FAILED:	// Failed to create shared data.
		return FALSE;
	}

	/////////////////////////////////////
	// Try to retrieve current user name.
	if (GetUserName(szUserName, &dwSize))
	{
		// If that worked, build name of the .INI file section
		// from the user name.
		m_strSettingSection = szUserName;
		m_strSettingSection += ".Settings";
	}
	else
	{
		// If it failed, use a default name for the .INI file section.
		m_strSettingSection = "Default.Settings";
	}

	// Is there anything left?
	if (!strFileName.IsEmpty())
		// Yes .. give dialog a copy of the command line.
		dlg.m_strFileName = strFileName;
	/////////////////////////////////////////////////////////
	// Set paths for .INI file and default save/load folder.
	// Retrieve the help file path.
	strFileName = m_pszHelpFilePath;
	// Find trailing backslash.
	iPos = strFileName.ReverseFind('\\');
	// Truncate to get just the path.
	strFileName = strFileName.Left(iPos);
	// Add name of our .INI file.
	strFileName += "\\TRADEKEYS.INI";
	// Free memory allocated by the CWinApp constructor.
	free((void*) m_pszProfileName);
	// Make a copy of the new .INI file path and install it.
	m_pszProfileName = _tcsdup((LPCTSTR) strFileName);
	// Try to retrieve save/load path from the .INI file.
	dlg.m_strSavePath = GetProfileString(m_strSettingSection, "SavePath", NULL);
	// Did we get it?
	if (dlg.m_strSavePath.IsEmpty())
		// No .. default to program directory.
		dlg.m_strSavePath = strFileName.Left(iPos);
	// Set dialog as our main window.
	m_pMainWnd = &dlg;
	// Launch the modal dialog.
	dlg.DoModal();
	// Done when the dialog is dismissed (we don't care how it was dismissed).
	return FALSE;
}

/////////////////////////
// Helpers for multiple instance handling.

// Locates or initializes file-mapped data.
// Returns value indicates whether we are the first instance,
// the second instance, or failure.
enum FILE_MAP_STATUS CTradeKeysApp::Init_Mapped_Data(void)
{
	FILE_MAP_STATUS result;
	// Try to open our file-mapped data object.
	if ((m_hMappedData = OpenFileMapping(FILE_MAP_WRITE, FALSE, "TradeKeys data")) == NULL)
	{	
		// If open failed, we must be the first instance, so
		result = MAP_FIRST;			// set flag,
		m_hMappedData =					// and try to create the object.
			CreateFileMapping(
				(HANDLE) 0xFFFFFFFFL,	// Use the system page file instead of a separate temporary file.
				NULL,					// Default security access.
				PAGE_READWRITE,			// Read-write access.
				0, sizeof(MY_SHARED_DATA),		// Reserve enough space to hold a UINT.
				"TradeKeys data");		// Create a named object so subsequent instances can find it.
	}
	else
		result = MAP_SECOND;
	// If object exists, try to create the necessary view of our mapped data.
	if (m_hMappedData != NULL)
		m_pSharedData = (MY_SHARED_DATA *) MapViewOfFile(m_hMappedData, FILE_MAP_WRITE, 0, 0, sizeof(UINT));
	// If unable to access mapped data, return failure code to caller.
	if (m_hMappedData == NULL || m_pSharedData == NULL)
	{
		::MessageBox(NULL, "Unable to create or access shared data.", "VolumeSet", MB_ICONSTOP);
		result =  MAP_FAILED;
	}
	// If we're the first instance, initialize the data.
	if (result == MAP_FIRST)
	{
		m_pSharedData->hWndMain = NULL;
		memset(m_pSharedData->pszFileName, 0, MAX_PATH);
	}
	return result;
}

